import { Injectable, Logger } from "@nestjs/common";
import { AgentService } from "../askar/agent.service";
import {
  ProofReqDto,
  CloudEventDto,
  CONNECTION_ACCEPT,
  CONNECTION_CREATE,
  CONNECTION_GET,
  CONNECTION_LIST,
  CreateCredentialDefinitionRequsetDto,
  CreateInvitationResponseDto,
  CreateSchemaRequestDto,
  CRED_DEF_CREATE,
  CRED_ISSUE,
  CRED_LIST,
  CRED_OFFER_ACCEPT,
  CRED_OFFER_LIST,
  IssueCredentialRequestDto,
  IssueProofRequestDto,
  MakeBasicMessageRequestDto,
  MESSAGE_MAKE,
  PROOF_ACCEPT,
  PROOF_ISSUE,
  PROOF_LIST,
  PROOF_GET,
  SCHEMA_CREATE,
  SCHEMA_GET,
  SCHEMA_LIST,
  PROOF_DECLINE,
  IdReqDto,
} from "@ocm-engine/dtos";

@Injectable()
export class EventHandlerService {
  private readonly logger = new Logger(EventHandlerService.name);

  constructor(private readonly agentService: AgentService) {}

  async handle<T>(event: CloudEventDto<T>) {
    let data;
    let dto;
    switch (event.type) {
      case CONNECTION_CREATE:
        data = await this.agentService.createInvitation();
        break;
      case CONNECTION_ACCEPT:
        dto = event.data as CreateInvitationResponseDto;
        data = await this.agentService.acceptInvitation(dto.invitationUrl);
        break;

      case CONNECTION_LIST:
        data = await this.agentService.fetchConnections();
        break;

      case CONNECTION_GET:
        dto = event.data as IdReqDto;
        data = await this.agentService.getConnectionById(dto.id);
        break;

      case SCHEMA_CREATE:
        dto = event.data as CreateSchemaRequestDto;
        data = await this.agentService.createSchema(dto);
        break;
      case SCHEMA_LIST:
        data = await this.agentService.fetchSchemas();
        break;

      case SCHEMA_GET:
        dto = event.data as IdReqDto;
        data = await this.agentService.getSchemaById(dto.id);
        break;
      case CRED_DEF_CREATE:
        data = await this.agentService.createCredentialDefinition(
          event.data as CreateCredentialDefinitionRequsetDto,
        );
        break;

      case CRED_ISSUE:
        data = await this.agentService.issueCredential(
          event.data as IssueCredentialRequestDto,
        );
        break;

      case CRED_LIST:
        data = await this.agentService.credentials();
        break;

      case CRED_OFFER_LIST:
        data = await this.agentService.credentialByStatedOfferReceived();
        break;

      case CRED_OFFER_ACCEPT:
        dto = event.data as IdReqDto;
        data = await this.agentService.acceptCredential(dto.id);
        break;

      case PROOF_ISSUE:
        dto = event.data as IssueProofRequestDto;
        data = await this.agentService.issueProof(dto);
        break;

      case PROOF_LIST:
        data = await this.agentService.proofs();
        break;

      case PROOF_GET:
        dto = event.data as IdReqDto;
        data = await this.agentService.getProofById(dto.id);
        break;

      case PROOF_ACCEPT:
        data = await this.agentService.acceptProof(event.data as ProofReqDto);
        break;

      case PROOF_DECLINE:
        dto = event.data as IdReqDto;
        data = await this.agentService.declineProofRequest(dto.id);
        break;

      case MESSAGE_MAKE:
        dto = event.data as MakeBasicMessageRequestDto;
        data = await this.agentService.sendMessage(dto);
        break;
    }

    event.data = data;
    return event;
  }
}
